// Problem: Serial Numbers
// Author: Rudy van Vliet
// Date: 3 September 2010

// Solution with dynamic programming: iterate over guitars 1..N.
// At any iteration i, maintain an array int MaxNrGuitars[M],
// where MaxNrGuitars[r] is the maximum number of guitars from 1..i that
// add up to r (mod M).
// Time complexity: O(N*M).
// Space complexity: O(M).
// Program seems to be correct.

#include <iostream>
using namespace std;

const int MaxN = 500;       // maximum number of guitars
const int MaxM = 100000;    // maximum magic number M
const int MaxSNr = 100000;  // maximum serial number
const char EOL = '\n';
int MaxNrGuitars[2][MaxM];  // in fact two arrays MaxNrGuitars:
                            // one with values up to guitar i-1
                            // and one with values up to guitar i
int SNr[MaxN+1];            // serial numbers of guitars 1..N
bool SNrRead[MaxSNr+1];     // has a serial number been read already?

//****************************************************************************

bool CheckParameters (int N, int M, int Inst)
 // Check values of parameters.
{ bool OK;

  OK = true;

  if ((N<1) || (N>MaxN))
  { cout << "incorrect value for N at instance " << Inst << ": "
          << N << EOL;
    OK = false;
  }

  if ((M<1) || (M>MaxM))
  { cout << "incorrect value for M at instance " << Inst << ": "
          << M << EOL;
    OK = false;
  }

  return OK;

}  // CheckParameters

//****************************************************************************

int ComputeMaxNrGuitars (int N, int M)
 // compute the maximum number of guitars for which the sum
 // of the serial numbers is a multiple of M
{ int old, current,  // the two arrays
      r,             // remainder module M
      oldr, newr,
      i,
      nr,
      NewNrGuitars;

  // initialize array MaxNrGuitars
  old = 1;
  current = 0;
  MaxNrGuitars[current][0] = 0;
  for (r=1;r<M;r++)
    MaxNrGuitars[current][r] = -1;  // not possible with 0 guitars

  for (i=1;i<=N;i++)
  {   // switch arrays
    old = 1 - old;
    current = 1 - current;

    nr = SNr[i];
    oldr = 0;
    newr = (oldr + nr) % M;
    while (oldr<M)
    { if (MaxNrGuitars[old][oldr] == -1)
      { // We cannot reach newr with guitar i.
        // We may have reached newr already without guitar i.
        MaxNrGuitars[current][newr] = MaxNrGuitars[old][newr];
      }
      else
        // We can reach newr with guitar i.
        // Can we also reach it without guitar i?
      { NewNrGuitars = MaxNrGuitars[old][oldr] + 1;
        if (NewNrGuitars > MaxNrGuitars[old][newr])
          MaxNrGuitars[current][newr] = NewNrGuitars;
        else
          MaxNrGuitars[current][newr] = MaxNrGuitars[old][newr];
      }

      oldr ++;
      newr = (newr+1) % M;
    }  // while

  }  // for i

  return (MaxNrGuitars[current][0]);

}  // ComputeMaxNrGuitars

//****************************************************************************

int main ()
{ int NInst, Inst;
  int N, M,
      nr,
      i,
      NrGuitars;
  bool OK;

  cin >> NInst;
  for (Inst=1;Inst<=NInst;Inst++)
  { cin >> N >> M;

    OK = CheckParameters (N, M, Inst);

    for (nr=0;nr<=MaxSNr;nr++)
      SNrRead[nr] = false;

    for (i=1;i<=N;i++)
    { cin >> nr;  // continue reading input, even if some value was not OK

      if ((nr<0) || (nr>MaxSNr))
      { cout << "incorrect serial number at instance " << Inst << ": "
             << nr << EOL;
        OK = false;
      }
      else  // in principle, a correct serial number
      { if (SNrRead[nr])
        { cout << "double occurrence of serial number at instance " << Inst
               << ": " << nr << EOL;
          OK = false;
        }
        else
          SNrRead[nr] = true;
      }

      if (i<=MaxN)  // is always true, provided N<=MaxN
        SNr[i] = nr;
    }

    if (OK)
    { NrGuitars = ComputeMaxNrGuitars (N, M);
      cout << NrGuitars << EOL;
    }

  }  // for Inst
  
  return 0;

}
