// Problem: Equal Is Not Really Equal
// Author: Rudy van Vliet
// Date: 19 August 2010

// Solution is based on representation of string as a graph.
//   We do not use the three-neighbours property in this solution,
//   so this solution may be a bit slower than `equalRudy.cc'.
// Time complexity: O(N*M*M), where N is the length of the string,
//   and M is the size of the alphabet

#include <iostream>
#include <cstring>
using namespace std;

const int MaxLength = 10000;  // maximum length of string
const char EOL = '\n';
const int NrDifferentChars = 26;
const int NoIndex = -1;  // default index for wrong character
int NrArcs[NrDifferentChars][NrDifferentChars];
     // number of arcs from one character to another
int NrIncArcs[NrDifferentChars];  // number of incoming arcs for character
bool Visited[NrDifferentChars];  // for DFS
struct AdjElt
{ int endnode;
  int nr;  // number of arcs to endnode
  AdjElt *next;
};
typedef AdjElt* AdjEltptr;
AdjEltptr AdjList[NrDifferentChars];

//****************************************************************************

int IndexOfChar (char ch)
 // return index of character `ch'
 //   0..25   for `A'..`Z'
{ int Index;

  if ((ch >= 'A') && (ch <= 'Z'))  // indeed an upper case letter
    Index = 0 + ((int) ch) - ((int) 'A');
  else
  { Index = NoIndex;
    cout << "Wrong character: " << ch << EOL;
  }

  return Index;

}  // IndexOfChar

//****************************************************************************

int CountNrArcs (char *str)
 // return: 0, if we can conclude already that str is unique
 //           (if all characters in str are equal)
 //         1, if we can conclude already that str is not unique
 //           (if str contains at least two different characters,
 //            and the first character of str equals the last character)
 //         2, otherwise
{ int  Index1, Index2,
      length, pos;
  bool differentcharacters;
  char firstchar;

  for (Index1=0;Index1<NrDifferentChars;Index1++)
    for (Index2=0;Index2<NrDifferentChars;Index2++)
      NrArcs[Index1][Index2] = 0;

  firstchar = str[0];
  Index1 = IndexOfChar (firstchar);
  differentcharacters = false; // we have not seen different characters, yet

  length = strlen (str);
  for (pos=1;pos<=length-1;pos++)
  { if (str[pos] != firstchar)
      differentcharacters = true;
    Index2 = IndexOfChar (str[pos]);
    NrArcs[Index1][Index2] ++;
    Index1 = Index2;
  }

  if (!differentcharacters)  // all characters are equal, hence unique
  {
//    cout << "Unique because all characters are equal.\n";
    return 0;
  }
  else  // different characters
    if (str[0]==str[length-1])  // first character equals last character,
    {
//      cout << "Not unique, because different characters with begin=end.\n";
      return 1;                 // hence not unique
    }
    else
      return 2;  // not yet determined: maybe unique, maybe not unique

}  // CountNrArcs

//****************************************************************************

void CreateAdjListNrIncArcs ()
 // transform adjacency matrix in adjacency list
 // and fill array NrIncArcs
{ int i, j;
  AdjElt *oldelt;

  for (i=0;i<NrDifferentChars;i++)
    AdjList[i] = NULL;

  for (j=0;j<NrDifferentChars;j++)
    NrIncArcs[j] = 0;

  for (i=0;i<NrDifferentChars;i++)
  { for (j=0;j<NrDifferentChars;j++)
    { if (NrArcs[i][j] > 0)
      { oldelt = AdjList[i];
        AdjList[i] = new AdjElt;
        AdjList[i]-> endnode = j;
        AdjList[i]-> nr = NrArcs[i][j];
        AdjList[i]-> next = oldelt;
        NrIncArcs[j] += NrArcs[i][j];
      }
    }
  }

}  // CreateAdjListNrIncArcs

//****************************************************************************

void PrintAdjList ()
{ int i;
  AdjElt* elt;

  for (i=0;i<NrDifferentChars;i++)
    if (AdjList[i] != NULL)
    { cout << i << ":";
      elt = AdjList[i];
      do
      { cout << " (" << elt->endnode << "," << elt->nr << ")";
        elt = elt -> next;
      } while (elt != NULL);
      cout << EOL;
    }  // if

}  // PrintAdjList

//****************************************************************************

void DFS (int i, int &ToBeVisited)
 // perform DFS from i, visiting only nodes thar are not yet visited
{ AdjElt *elt;
  int j;

  elt = AdjList[i];
  while ((elt!=NULL) && (ToBeVisited>0))
  {
    j = elt->endnode;
    if ( (elt->nr > 0) && (!(Visited[j])) )
      // we can visit j and we have not done that before
    { Visited[j] = true;
      ToBeVisited --;
      DFS (j, ToBeVisited);
    }
    elt = elt-> next;
  }  // while
  
}  // DFS

//****************************************************************************

bool CheckDFS (int i)
 // check if all nodes j with NrIncArcs[j]>0 can still be reached from node i
{ int ToBeVisited,  // number of nodes that we have to visit
      j;

  ToBeVisited = 0;
  for (j=0;j<NrDifferentChars;j++)
  { Visited[j] = false;
    if (NrIncArcs[j]>0)  // we should visit j
      ToBeVisited ++;
  }

  DFS (i, ToBeVisited);

  if (ToBeVisited==0)
    return true;
  else  // we cannot visit all nodes
    return false;

}  // CheckDFS

//****************************************************************************

void DeleteAdjList ()
 // delete the adjacency list (free memory)
{ int i;
  AdjElt *elt, *nextelt;

  for (i=0;i<NrDifferentChars;i++)
  { elt = AdjList[i];
    while (elt != NULL)
    { nextelt = elt -> next;
      delete elt;
      elt = nextelt;
    }
    AdjList[i] = NULL;
  }

}  // DeleteAdjList


//****************************************************************************

bool CheckUnique (char *str)
 // pre: str contains at least two different characters,
 //      and first character of str is not equal to last character
{ int Index1, Index2, k,
      length, pos;
  AdjElt *elt, *Index2elt;
  bool unique,
       OK;

  CreateAdjListNrIncArcs ();
//  PrintAdjList ();

  unique = true;
  length = strlen (str);
  pos = 0;
  Index1 = IndexOfChar (str[pos]);
  while (unique && (pos<length-1))
  { pos ++;
    Index2 = IndexOfChar (str[pos]);

    elt = AdjList[Index1];

      // check if we can go to another node than Index2
    while (unique && (elt != NULL))
    {
      k = elt-> endnode;

      if (k==Index2)  // remember this elt
        Index2elt = elt;
      else  // different endnode k
      { if (elt -> nr > 0)  // we can still go there
        {   // remove arc temporarily
          (elt -> nr) --;
          NrIncArcs[k] --;

          if (elt -> nr == 0)
            OK = CheckDFS (k); // check if every (remaining) node is still
                               // reachable from k
                    // We will perform this DFS for at most two neighbours k.
                    // If Index1 has two neighbours different from Index2,
                    // then for at least one of them CheckDFS will
                    // yield true (hence, the string is not unique).
          else  // for sure, every (remaining) node is still reachable from k
            OK = true;

          if (OK)
            unique = false;

            // restore arc
          (elt -> nr) ++;
          NrIncArcs[k] ++;
        }
        // else: we cannot go there

      }  // different endnode

      elt = elt -> next;
    }  // inner while

    if (unique)  // go to Index2
    { (Index2elt -> nr) --;
      NrIncArcs[Index2] --;
      Index1 = Index2;
    }
  }  // outer while
  
  DeleteAdjList ();

  return unique;

}  // CheckUnique

//****************************************************************************

int main ()
{ int NInst, Inst;
  char OrigString[MaxLength+1];
  int code;

  cin >> NInst;
  for (Inst=1;Inst<=NInst;Inst++)
  { cin >> OrigString;  // does not work for strings containing spaces, ...

    code = CountNrArcs (OrigString);
    if (code == 0)
      cout << "unique\n";
    else
      if (code == 1)
        cout << "not unique\n";
      else  // code == 2
      { if (CheckUnique (OrigString))
          cout << "unique\n";
        else
          cout << "not unique\n";
      }


  }  // for Inst
  
  return 0;

}
