// Problem: The great cleanup
// Author: Rudy van Vliet
// Version of `cleanupRudy.cc' which does not delete the Trie
// after a test case. Meant to check for memory capacity problems.
// Date: 10 September 2010

// Program seems to work well.
// Time complexity: O(N), where N is the total number of characters in
//   the filenames
// Stores the file names from the input in a Trie, and traverses the Trie
// to count the number of rm-commands required.
// To save memory space, we give every TrieNode no more than 63 children
// (instead of 256). This requires some indexing, but it works.

#include <iostream>
#include <cstring>
using namespace std;

const int MaxN1 = 1000;
const int MaxN2 = 1000;
const int MaxFileNameLength = 20;
const int StringLength = 1024;
const int NrDifferentChars = 63;
const int NoIndex = -1;  // default index for wrong character
const int NoFile = 0;
const int MustBeRemoved = 1;
const int MustStay = 2;
const char EOL = '\n';
const char EOS = '\0';

struct TrieNode
{ int Status;   // 0  no file with this name
                // 1  file must be removed
                // 2  file must stay
  bool RemoveInSubTrie,
       StayInSubTrie;
  TrieNode *Children[NrDifferentChars];
};
int Indices[256];
char Chars[NrDifferentChars];

//****************************************************************************

int IndexOfChar (char ch)
 // return index of character `ch'
 //   0       for `.'
 //   1..10   for `0'..`9'
 //   11..36  for `A'..`Z'
 //   37..62  for `a'..`z'
{ int Index;

  if (ch == '.')  // a period
    Index = 0;
  else
    if ((ch >= '0') && (ch <= '9'))  // a digit
      Index = 1 + ((int) ch) - ((int) '0');
    else
      if ((ch >= 'A') && (ch <= 'Z'))  // an upper case letter
        Index = 11 + ((int) ch) - ((int) 'A');
      else
        if ((ch >= 'a') && (ch <= 'z'))  // a lower case letter
          Index = 37 + ((int) ch) - ((int) 'a');
        else
          Index = NoIndex;

  return Index;

}  // IndexOfChar

//****************************************************************************

void FillIndicesChars ()
 // fill arrays Indices and Chars with the right values/characters
{ int i,
      Index;

  for (i=0;i<=255;i++)
  { Index = IndexOfChar ((char) i);
    Indices[i] = Index;
    if (Index != NoIndex)
      Chars[Index] = (char) i;
  }

}  // FillIndicesChars

//****************************************************************************

bool ReadFile (char FileName[MaxFileNameLength+1], int Inst)
 // read a filename of at most MaxFileNameLength characters from the input
{ bool OK;
  char FileNameInInput[StringLength+1];

  OK = true;

  cin >> FileNameInInput;
  if ( ((int) (strlen (FileNameInInput))) > MaxFileNameLength)
  { cout << "FileName too long at instance " << Inst << ": "
         << FileNameInInput << EOL;
    OK = false;
    strncpy (FileName, FileNameInInput, MaxFileNameLength);
    FileName[MaxFileNameLength] = EOS;  // is not done by strncpy
  }
  else
    strcpy (FileName, FileNameInInput);

  return OK;
    
}  // ReadFile

//****************************************************************************

TrieNode *NewTrieNode ()
{ TrieNode *node;
  int Index;

  node = new TrieNode;
  node -> Status = NoFile;
  node -> RemoveInSubTrie = false;
  node -> StayInSubTrie = false;
  for (Index=0;Index<NrDifferentChars;Index++)
    node -> Children[Index] = NULL;
  
  return node;

}  // NewTrieNode

//****************************************************************************

bool StoreInTrie (TrieNode* &root, char FileName[MaxFileNameLength+1],
                  bool Remove, int &NrTrieNodes)
 // Store FileName in the Trie
{ TrieNode *walker;
  bool OK;
  int len, pos,
      Index;
  char ch;

  if (root==NULL)
  { root = NewTrieNode ();
    NrTrieNodes ++;
  }

  walker = root;
  if (Remove)
    walker -> RemoveInSubTrie = true;
  else
    walker -> StayInSubTrie = true;

  OK = true;
  len = strlen (FileName);

  if (len==0)  // empty filename
  { cout << "Empty filename\n";  // we can continue with this function anyway
    OK = false;
  }

  pos = 0;

  while (OK && (pos<len))
   // invariant: walker != NULL
  { ch = FileName[pos];
    Index = Indices [(int)ch];

    if (Index==NoIndex)
    { cout << "Wrong character in filename " << FileName << EOL;
      OK = false;
    }
    else
    { if (walker -> Children[Index]==NULL)
      { walker -> Children[Index] = NewTrieNode ();
        NrTrieNodes ++;
      }

      walker = walker -> Children[Index];
      if (Remove)
        walker -> RemoveInSubTrie = true;
      else
        walker -> StayInSubTrie = true;
    }

    pos ++;
  }  // while

  if (walker -> Status != NoFile)
  { cout << "Double filename " << FileName << EOL;
    OK = false;
  }
  else  // status was NoFile
  { if (Remove)
      walker -> Status = MustBeRemoved;
    else
      walker -> Status = MustStay;
  }

  return OK;

}  // StoreInTrie

//****************************************************************************

void TraverseTrie (TrieNode *root, int &NrRmCommands,
                  char CurrentPath[MaxFileNameLength+1], int len)
 // Recursively traverse the (sub)Trie with root `root',
 // and count the number of RmCommands required to remove exactly
 // the right files (with filenames in the (sub)Trie)
 // pre: root != NULL
{ TrieNode *child;
  int Index;
 
    // By the construction of the Trie, every subTrie contains at least
    // one endnode of a filename (which must or must not be removed).
    // Hence, for every node, either node->RemoveInSubTrie or
    // node->StayInSubTrie (or both).

  if (!(root -> RemoveInSubTrie))
       // All files must stay.
       // Stop traversal; we do not need any additional rm-command.
  {
  }
  else  // RemoveInSubTrie
  { if (!(root -> StayInSubTrie))
         // All files must be removed.
         // Stop traversal; one rm-command is sufficient.
    { NrRmCommands ++;
//      cout << "rm " << CurrentPath << "*" << EOL;
    }
    else  // RemoveInSubTrie && StayInSubTrie
    { if (root -> Status == MustBeRemoved)
        // an rm-command is required for this file (alone)
      { NrRmCommands ++;
//        cout << "rm " << CurrentPath << EOL;
      }
          // Continue the traversal with the children of root.
      for (Index=0;Index<NrDifferentChars;Index++)
      { child = root -> Children[Index];
        if (child != NULL)
        { CurrentPath[len] = Chars[Index];
          CurrentPath[len+1] = EOS;
          TraverseTrie (child, NrRmCommands, CurrentPath, len+1);
        }
      }  // for Index
    }  // RemoveInSubTrie && StayInSubTrie

  }

}  // TraverseTrie

//****************************************************************************

void DeleteTrie (TrieNode *root)
 // Recursively delete the (sub)Trie with root `root'.
 // pre: root != NULL
{ TrieNode *child;
  int Index;

  for (Index=0;Index<NrDifferentChars;Index++)
  { child = root -> Children[Index];
    if (child != NULL)
      DeleteTrie (child);
  }
  delete root;

}  // DeleteTrie

//****************************************************************************

int main ()
{ int NInst, Inst;
  int N1, N2,
      i;
  TrieNode *root;
  int NrTrieNodes;
  char FileName[MaxFileNameLength+1],
       CurrentPath[MaxFileNameLength+1];  // to store current path in Trie
                                          // during traversal;
                                          // used only to print rm-commands
  int len,  // length of CurrentPath
      NrRmCommands;
  bool OK, newOK;

  FillIndicesChars ();

  cin >> NInst;
  for (Inst=1;Inst<=NInst;Inst++)
  { OK = true;
    root = NULL;
    NrTrieNodes = 0;

    cin >> N1;
    if ((N1<1) || (N1>MaxN1))
    { cout << "incorrect value for N1 at instance " << Inst << ": "
            << N1 << EOL;
      OK = false;
    }

     // read names of files that must be removed
    for (i=1;i<=N1;i++)
     // continue reading input, even if value of N1 was not OK
    { newOK = ReadFile (FileName, Inst);
      if (!newOK)
        OK = false;
      newOK = StoreInTrie (root, FileName, true, NrTrieNodes);
      if (!newOK)
        OK = false;
    }

    cin >> N2;
    if ((N2<0) || (N2>MaxN2))
    { cout << "incorrect value for N2 at instance " << Inst << ": "
            << N2 << EOL;
      OK = false;
    }

     // read names of files that must stay
    for (i=1;i<=N2;i++)
     // continue reading input, even if value of N2 was not OK
    { newOK = ReadFile (FileName, Inst);
      if (!newOK)
        OK = false;
      newOK = StoreInTrie (root, FileName, false, NrTrieNodes);
      if (!newOK)
        OK = false;
    }

//    cout << "Number of TrieNodes = " << NrTrieNodes << EOL;

    if (OK)
    { NrRmCommands = 0;
      if (root!=NULL)
      { len = 0;
        CurrentPath[len] = EOS;
        TraverseTrie (root, NrRmCommands, CurrentPath, len);
      }

      cout << NrRmCommands << EOL;
    }

    if (root!=NULL)
    {
//      DeleteTrie (root);
      root = NULL;
    }

  }  // for Inst
  
  return 0;

}
