/*
 * BAPC 2010 Preliminaries
 * Jeroen Bransen
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>
#include <string.h>

struct Point {
    int X, Y;
};

/* -1 if negative, 1 otherwise */
int sign(int a) {
	return a < 0 ? -1 : 1;
}

/* A - B */
struct Point subtract(struct Point A, struct Point B) {
    struct Point ret;
    ret.X = A.X - B.X;
    ret.Y = A.Y - B.Y;
    return ret;
}

/* Dot product of A . B*/
int dot(struct Point A, struct Point B) {
    return A.X * B.X + A.Y * B.Y;
}

/* Cross product of A x B = |A||B|Sin(Theta) */
int cross(struct Point A, struct Point B) {
    return A.X * B.Y - A.Y * B.X;
}

/* Distance between A and B */
double distance(struct Point A, struct Point B) {
    return sqrt(dot(subtract(A, B), subtract(A, B)));
}

/* Is point C on line A-B? */
bool pointOnLine(struct Point A, struct Point B, struct Point C) {
    if (dot(subtract(C, B), subtract(B, A)) > 0) return false;
	if (dot(subtract(C, A), subtract(A, B)) > 0) return false;
    return cross(subtract(B, A), subtract(C, A)) == 0;
}

/* Do lines A-B and C-D intersect? */
bool intersect(struct Point A, struct Point B, struct Point C, struct Point D) {
	int d1, d2, det;
	struct Point E, F, G;
	E = subtract(B, A);
	F = subtract(D, C);
	G = subtract(A, C);
	det = cross(E, F);
    if (det == 0) { /* Parallel */
		return pointOnLine(A, B, C) || pointOnLine(A, B, D) || pointOnLine(C, D, A) || pointOnLine(C, D, B);
    } else {
		d1 = cross(F, G) * sign(det);
		d2 = cross(E, G) * sign(det);
		det = abs(det);
		return 0 <= d1 && d1 <= det && 0 <= d2 && d2 <= det;
    }
}

/*** The solution ***/
#define MAXN 100

struct Point points[MAXN * 2 + 2];
bool conn[MAXN*2+2][MAXN*2+2];
double dist[MAXN*2+2][MAXN*2+2];

void testcase() {
	int N, i, j, k;
	bool blocked;
	double newDist;

	/* Read input */
	scanf("%d %d %d %d\n", &points[0].X, &points[0].Y, &points[1].X, &points[1].Y);
	scanf("%d\n", &N);
	for(i=0;i<N;i++)
		scanf("%d %d %d %d\n", &points[i+2].X, &points[i+2].Y, &points[i+2+N].X, &points[i+2+N].Y);

	/* Set all connected edges */
	memset(conn, false, sizeof(conn));
	for(i=0;i<N*2+2;i++) {
		for(j=0;j<N*2+2;j++) {
			if(i==j) continue;
			blocked = false;
			for(k=0;k<N;k++) {
				if(k+2 != i && k+2 != j && k+2+N != i && k+2+N != j && intersect(points[i], points[j], points[k+2], points[k+2+N])) {
					blocked = true;
					break;
				}
			}
			if(!blocked) {
				conn[i][j] = true;
				dist[i][j] = distance(points[i], points[j]);
			}
		}
	}

	/* Floyd-Warshall for finding shortest distance */
	for(i=0;i<N*2+2;i++) {
		for(j=0;j<N*2+2;j++) {
			for(k=0;k<N*2+2;k++) {
				if(conn[j][i] && conn[i][k]) {
					newDist = dist[j][i]+dist[i][k];
					if(!conn[j][k] || newDist < dist[j][k]) {
						conn[j][k] = true;
						dist[j][k] = newDist;
					}
				}
			}
		}
	}

	/* Cornercase: Bessie is already at the food */
	if(points[0].X == points[1].X && points[0].Y == points[1].Y) {
		conn[0][1] = true;
		dist[0][1] = 0;
	}

	/* Print output */
	if(conn[0][1])
		printf("%.6lf\n", dist[0][1]);
	else
		printf("-1\n");
}

int main() {
	int t;
	scanf("%d\n", &t);
	while(t--) testcase();
	return 0;
}
