// Problem: Farmer John
// Author: Rudy van Vliet
// Date: 21 September 2010

// Program uses Dijkstra's algorithm to find shortest distance from
// Bessie to the food. 
// Because there are so many potential connections between the points, 
// it is no use maintaining a real priority queue with reachable points.
// However, we have all functions available to implement a priority queue.
// Program seems to work well.
//
// Program performs all tests on (long) integers.
// This works well, as long as (approximately)  (dx)^2 stays in the (long)
// integer domain.
// Time complexity: O(N^3)
//   because for every point cur we encounter with Dijkstra's algorithm,
//   we try to directly connect cur with every other point i,
//   which means that we check for every fence j, 
//   if the line segment between cur and i crosses fence j.

#include <iostream>
#include <iomanip>
#include <cmath>  // for sqrt
using namespace std;

const int MaxN = 100;
const long int MinX = -10000,
           MaxX = 10000,
           MinY = -10000,
           MaxY = 10000;
const char EOL = '\n';
const double DefaultDist = -1.0;

long int X[2*MaxN+2],  // location of Bessie, location of food,
     Y[2*MaxN+2];  // and two points per line segment
double Dist[2*MaxN+2];  // shortest distance known so far
int EltsInPriorQueue;   // number of elements in priority queue
bool InPriorQueue[2*MaxN+2];  // is point in priority queue?

//************************************************************************

bool ValueInBounds (long int x, long int minimum, long int maximum)
 // Check if x is between minimum and maximum (inclusive)
{
  if ((x>=minimum) && (x<=maximum))
    return true;
  else
  { cout << "Value " << x << " is not between bounds " << minimum
         << " and " << maximum << ".\n";
    return false;
  }

}  // ValueInBounds

//************************************************************************

long int MyAbs (long int x)
{
  if (x >= 0)
    return x;
  else
    return -x;

}  // MyAbs

//************************************************************************

bool OppositeSigns (long x, long y)
 // Check if x and y have opposite signs: one positive and the other negative
{
  if ( ((x<0) && (y>0)) || ((x>0) && (y<0)) )
    return true;
  else
    return false;

}  // OppositeSigns

//************************************************************************

int DetermineSignQuotient (long int numerator, long int denominator)
  // Determine `sign' of quotient numerator/denominator
  // return: -2 if quotient < 0
  //         -1 if quotient = 0
  //         0  if 0 < quotient < 1
  //         1  if quotient = 1
  //         2  if quotient > 1
  // Pre: denominator != 0
{ int sign;

//   if (numerator*denominator < 0)  // opposite signs,
// This runs out of the (long) integer domain.
// Apparently long int is not really long int...
  if (OppositeSigns (numerator, denominator))
    sign = -2;                    // hence, quotient < 0
  else
    if (numerator==0)
      sign = -1;
    else  // both numerator and denominator are non-zero and they have
          // the same sign
      if (MyAbs (numerator) < MyAbs(denominator))
        sign = 0;
      else
        if (numerator==denominator)
          sign = 1;
        else
          if (MyAbs (numerator) > MyAbs (denominator))
            sign = 2;
          else  // should be impossible, but for security
          { cout << "Incorrect sign!\n";
            sign = 3;
          }

  return sign;

}  // DetermineSignQuotient

//************************************************************************

int DetermineLambda (long int x1, long int y1, long int dx1, long int dy1, 
                     long int x3, long int y3)
 // Determine value for lambda in equation
 //     (x3,y3) = (x1,y1) + lambda (dx1,dy1)
 // return: -2 if lambda < 0
 //         -1 if lambda = 0 
 //         0  if 0 < lambda < 1
 //         1  if lambda = 1
 //         2  if lambda > 1
 // Pre: - (dx1,dy1) != (0,0) 
 //      - (x3,y3) is on line (x1,y1) + lambda (dx1,dy1)
{ long int helpdx, helpdy;
  int lambda;

  if (dx1!=0)
  { helpdx = x3-x1;
    lambda = DetermineSignQuotient (helpdx, dx1);
  }
  else  // dx1=0, hence x3=x1.
        // Then look at y-coordinates.
        // By assumption, dy1!=0
  { helpdy = y3-y1;
    lambda = DetermineSignQuotient (helpdy, dy1);
  }
 
  return lambda;

}  // DetermineLambda

//************************************************************************

int CheckLineCrossing (long int x1, long int y1, long int x2, long int y2,
                       long int x3, long int y3, long int x4, long int y4)
 // Check if two line segments cross or touch each other
 // return: 4 if the two line segments cross each other
 //             (they must be non-parallel then)
 //         3 if the two line segments touch each other. That is, they
 //             have (at least) a common point, but they do not cross 
 //             each other (they may be parallel then)
 //         2 if the two line segments are colinear, but they do not
 //             touch each other
 //         1 otherwise (they may be parallel but on different lines,
 //             or non-parallel and not touching each other)
 // Pre: both line segments have positive length.
{ long int dx1, dy1,
       dx3, dy3,
       helpdx, helpdy,
       denominator,
       numeratorlambda, numeratormu;
  int  lambda3, lambda4,
       lambda, mu,
       value;

  dx1 = x2 - x1;
  dy1 = y2 - y1;
  dx3 = x4 - x3;
  dy3 = y4 - y3;
  helpdx = x3 - x1;  // line segment from (x1,y1) to (x3,y3)
  helpdy = y3 - y1;

  denominator = -dy3*dx1 + dy1*dx3;

  if (denominator==0)   // the two line segments are parallel
  { 
    if (dx1*helpdy == dy1*helpdx)
      // the two line segments are even colinear
    { lambda3 = DetermineLambda (x1, y1, dx1, dy1, x3, y3);
      lambda4 = DetermineLambda (x1, y1, dx1, dy1, x4, y4);
      if (lambda3*lambda4 == 4)
        // Either -2*-2 or 2*2.
        // Hence, (x3,y3) and (x4,y4) are at the same side of line segment
        // between (x1,y1) and (x2,y2)
        value = 2;
      else  // on different sides, or one of the points is even
            // at the line segment
        value = 3;
    }
    else  // parallel, but not colinear
      value = 1;
  }  // if parallel

  else  // not parallel (i.e., denominator != 0)
  {  // Determine intersection point of the two lines, as solution to
     // equation
     //   (x1,y1) + lambda (dx1,dy1) = (x3,y3) + mu (dx3,dy3)
     numeratorlambda = helpdy*dx3 - helpdx*dy3;
     lambda = DetermineSignQuotient (numeratorlambda, denominator);
     numeratormu = helpdy*dx1 - helpdx*dy1;
     mu = DetermineSignQuotient (numeratormu, denominator);

     if ((lambda==0) && (mu==0))  // the two line segments cross
       value = 4;
     else
       if ((lambda>=-1) && (lambda<=1) && (mu>=-1) && (mu<=1))
         value = 3;
       else  // lambda or mu (or both) is -2 or 2
         value = 1;
  }

  return value;

}  // CheckLineCrossing

//************************************************************************

int CheckPointOnLine (long int x1, long int y1, long int x2, long int y2,
                      long int x3, long int y3)
 // Check if (x3,x4) is on line segment between (x1,y1) and (x2,y2)
 // return 0: if (x3,x4) is not even on line through (x1,y1) and (x2,y2)
 //        1: if (x3,x4) is on line through (x1,y1) and (x2,y2),
 //             but not an line segment between (x1,y1) and (x2,y2)
 //        2: if (x3,x4) on line segment between (x1,y1) and (x2,y2)
 // Pre: (x1,y1) != (x2,y2)
{ long int dx1, dy1,
       helpdx, helpdy,
       denominator;
  int  lambda,
       value;

  dx1 = x2 - x1;
  dy1 = y2 - y1;
  helpdx = x3 - x1;
  helpdy = y3 - y1;

  denominator = -helpdy*dx1 + dy1*helpdx;

  if (denominator==0)   // (x3,x4) is on line through (x1,y1) and (x2,y2)
  { lambda = DetermineLambda (x1, y1, dx1, dy1, x3, y3);
    if ((lambda>=-1) && (lambda<=1))  // on line segment
      value = 2;
    else  // not on line segment
      value = 1;
  }  
  else  // denominator != 0, hence (x3,x4) is not on line through
        // (x1,y1) and (x2,y2)
    value = 0;

  return value;

}  // CheckPointOnLine

//************************************************************************

bool CheckInput (int N, int Inst)
  // Check 
  // - if Bessie and the food are not at a fence
  // - and if the fences do not touch (or cross) each other
{ bool OK;
  int i, j;

  OK = true;

  for (i=1;i<=N;i++)
  { if (CheckPointOnLine (X[2*i], Y[2*i], X[2*i+1], Y[2*i+1], X[0], Y[0])
        == 2)
    { cout << "Bessie is at fence " << i << " at Inst=" << Inst << ".\n";
      OK = false;
    }

    if (CheckPointOnLine (X[2*i], Y[2*i], X[2*i+1], Y[2*i+1], X[1], Y[1])
        == 2)
    { cout << "Food is at fence " << i << " at Inst=" << Inst << ".\n";
      OK = false;
    }

    for (j=i+1;j<=N;j++)
    { if (CheckLineCrossing (X[2*i], Y[2*i], X[2*i+1], Y[2*i+1],
                             X[2*j], Y[2*j], X[2*j+1], Y[2*j+1]) >= 3)
      { cout << "Fences " << i << " and " << j << " at Inst=" << Inst
             << " cross or touch each other.\n";
        OK = false;
      }
    }  // for j

  }  // for i

  return OK;

}  // CheckInput

//************************************************************************

void InitPriorQueue (int N)
 // Initialize priority queue.
{ int i;

  // Because there are so many potential connections between the points,
  // it is no use maintaining a real priority queue with reachable points.

  for (i=0;i<=2*N+1;i++)
    InPriorQueue[i] = false;
  EltsInPriorQueue = 0;

}  // InitPriorQueue

//************************************************************************

void AddToPriorQueue (int i, int N)
 // Add i to priority queue.
{
  // Because there are so many potential connections between the points,
  // it is no use maintaining a real priority queue with reachable points.

  if (! (InPriorQueue[i]) )
  { InPriorQueue[i] = true;
    EltsInPriorQueue ++;
  }
  else
    cout << "Element " << i << " is already in priority queue.\n";

}  // AddToPriorQueue

//************************************************************************

int RemoveMinFromPriorQueue (int N)
 // Remove minimum from priority queue, and return this i
{ int Mini, i;
  double MinDist;

  Mini = -1;

  if (EltsInPriorQueue > 0)
  {
    for (i=0;i<=2*N+1;i++)
    { if (InPriorQueue[i])
      { if (Mini==-1)  // first element in priority queue
        { MinDist = Dist[i];
          Mini = i;
        }
        else
        { if (Dist[i]<MinDist)
          { MinDist = Dist[i];
            Mini = i;
          }
        }
      }  // InPriorQueue
    } // for i

    InPriorQueue[Mini] = false;
    EltsInPriorQueue --;
    return Mini;
  }
  else
  { cout << "Priority queue is already empty.\n";
    return Mini;
  }
  
}  // RemoveMinFromPriorQueue

//************************************************************************

void UpdatePriorQueue (int i, int N)
 // Update priority queue, because point i has a new (lower) value
{
  // Because there are so many potential connections between the points,
  // it is no use maintaining a real priority queue with reachable points.
  
  if (! (InPriorQueue[i]) )
    cout << "Point i is not in priority queue.\n";
  
}  // UpdatePriorQueue

//************************************************************************

bool PriorQueueIsEmpty (int N)
{
  if (EltsInPriorQueue==0)
    return true;
  else
    return false;

}  // PriorQueueIsEmpty

//************************************************************************

double EuclidDistance (long int x1, long int y1, long int x2, long int y2)
{ double dx1, dy1;

  dx1 = x2 - x1;
  dy1 = y2 - y1;

  return sqrt (dx1*dx1 + dy1*dy1);

}  // EuclidDistance

//************************************************************************

double ComputeDistance (int N)
{ int i,
      cur,
      j;
  double distance;
  bool InTree[2*MaxN+2],
       Direct;

  for (i=0;i<=2*N+1;i++)
  { Dist[i] = DefaultDist;
    InTree[i] = false;
  }

  InitPriorQueue (N);

  i = 0;
  Dist[i] = 0.0;
  AddToPriorQueue (i, N);

  do
  { cur = RemoveMinFromPriorQueue (N);
    InTree[cur] = true;

    if (cur != 1)
    {  // which points can we reach directly from cur?
      for (i=0;i<=2*N+1;i++)
      { if (! (InTree[i] ) )  // check if we can reach i directly from cur
        { Direct = true;
          for (j=1;Direct&&(j<=N);j++)
          { if (CheckLineCrossing (X[cur], Y[cur], X[i], Y[i],
                                   X[2*j], Y[2*j], X[2*j+1], Y[2*j+1]) == 4)
              Direct = false;
          }

          if (Direct)
          { distance = Dist[cur] + EuclidDistance (X[cur], Y[cur], X[i], Y[i]);
            if (Dist[i] == DefaultDist)
            { Dist[i] = distance;
              AddToPriorQueue (i, N);
            }
            else
              if (distance < Dist[i])
              { Dist[i] = distance;
                UpdatePriorQueue (i, N);
              }
              // else: new distance is longer than known distance
          }  // Direct

        }  // not InTree[i]
      }  // for i        
    }  // cur != 1

  } while ((cur != 1) && !(PriorQueueIsEmpty(N) ));

  if (cur == 1)
    return Dist[cur];
  else
  { cout << "Bessie cannot reach the food!\n";
    return DefaultDist;
  }

}  // ComputeDistance

//************************************************************************

int main ()
{ int NInst, Inst;
  int N, i;
  long int x1, y1, x2, y2;
  bool OK;
  double distance;

  cin >> NInst;
  for (Inst=1;Inst<=NInst;Inst++)
  { OK = true;

    cin >> X[0] >> Y[0] >> X[1] >> Y[1];  // location of Bessie,
                                          // and location of food
    OK = ValueInBounds (X[0], MinX, MaxX);
    if (OK)
      OK = ValueInBounds (Y[0], MinY, MaxY);
    if (OK)
      OK = ValueInBounds (X[1], MinX, MaxX);
    if (OK)
      OK = ValueInBounds (Y[1], MinY, MaxY);

    cin >> N;
    if ((N<0) || (N>MaxN))
    { cout << "Incorrect value for N at Inst=" << Inst << ": " << N << EOL;
      OK = false;
    }

    for (i=1;i<=N;i++)
      // proceed reading input, even if some value was not correct
    {
      cin >> x1 >> y1 >> x2 >> y2;
      if (OK)
        OK = ValueInBounds (x1, MinX, MaxX);
      if (OK)
        OK = ValueInBounds (y1, MinY, MaxY);
      if (OK)
        OK = ValueInBounds (x2, MinX, MaxX);
      if (OK)
        OK = ValueInBounds (y2, MinY, MaxY);

      if (OK)  // store values
      { X[2*i] = x1; 
        Y[2*i] = y1;
        X[2*i+1] = x2; 
        Y[2*i+1] = y2;
      }
    }  // for i
    
    if (OK)
      OK = CheckInput (N, Inst);

    if (OK)
    { distance = ComputeDistance (N);
      cout << setiosflags (ios::fixed) << setprecision (6) << distance << EOL;
    }

  }  // for Inst

  return 0;

}
