// Problem: My cousin Obama
// Author: Rudy van Vliet
// Date: 16 September 2010
// Version of `obamaRudy.cc' where every `cin' is substituted by `scanf'.

// Uses a double ended queue (in an array) to store ancestors
// that have to be further examined.
// Linear time complexity.
// Program seems to be correct.

#include <iostream>
using namespace std;

const int MinN = 2;
const int MaxN = 100000;
const int MinID = 1;    // MinID must be non-negative and smaller than MinN
const int Unknown = 0;  // default ID number, if the father or the mother
                        // of a person is unknown
                        // of course, this default value must not be
                        // in the interval [MinID..MaxN]
const int UnknownGender = 0;
const int Man = 1;
const int Woman = 2;
const char EOL = '\n';

int Father[MaxN+1], Mother[MaxN+1],
    Gender[MaxN+1],    // 0: unknown   1: man   2: woman
    DEQueue[2*MaxN+1], // large enough to grow to the left or to grow
                       // to the right, if we start in the middle
    FirstIndex, LastIndex,  // of entries in DEQueue
    Distance[MaxN+1],  // shortest distance from A0 to i
                       // (i.e., the smallest number of women on a line
                       // of ancestry from A0 to i, if any such line has
                       // been found)
    ChildToA0[MaxN+1]; // child of i that is responsible for Distance[i]

//****************************************************************************

bool CheckParameters (int N, int A0, int B0, int Inst)
 // Check values of parameters.
{ bool OK;

  OK = true;

  if ((N<MinN) || (N>MaxN))
  { cout << "incorrect value for N at instance " << Inst << ": "
          << N << EOL;
    OK = false;
  }

  if ((A0<MinID) || (A0>N))
  { cout << "incorrect value for A0 at instance " << Inst << ": " << A0 << EOL;
    OK = false;
  }

  if ((B0<MinID) || (B0>N))
  { cout << "incorrect value for B0 at instance " << Inst << ": " << B0 << EOL;
    OK = false;
  }

  if (A0==B0)
  { cout << "A0 and B0 are equal at instance " << Inst << ": " << A0 << EOL;
    OK = false;
  }

  return OK;

}  // CheckParameters

//****************************************************************************

void PrintTopOrder (int MinID, int N, int Index, int Order[MaxN+1])
 // Print topological order, as stored in array Order
 // (for as much as we could determine).
{ int i;

  if (Index>N)  // complete topological order
    cout << "A topological order is:\n";
  else
  { cout << "We could not determine a complete topological order.\n";
    cout << "We only found:\n";
  }
    
  for (i=MinID;i<Index;i++)
    cout << ' ' << Order[i];
  cout << EOL;

}  // PrintTopOrder

//****************************************************************************

bool CheckDAG (int N, int Inst)
 // Check if ancestor information describes Directed Acyclic Graph.
{ int IncArcs[MaxN+1],
      i, f, m,
      Order[MaxN+1],  // to store persons in topological order
                      // at positions MinID..N
      Index,          // position in array Order
      ZeroQueue[MaxN+1],  // Queue with persons with 0 incoming arcs
      FirstInQ, LastInQ;  // First/Last position of actual Queue in ZeroQueue

  for (i=MinID;i<=N;i++)
    IncArcs[i] = 0;

  for (i=MinID;i<=N;i++)
  { f = Father[i];
    if (f!=Unknown)
      IncArcs[f] ++;

    m = Mother[i];
    if (m!=Unknown)
      IncArcs[m] ++;
  }

  FirstInQ = 1;
  LastInQ = 0;
  for (i=MinID;i<=N;i++)
  { if (IncArcs[i] == 0)
    { LastInQ ++;
      ZeroQueue[LastInQ] = i;
    }
  }

  Index = MinID;
  while ((Index<=N) && (FirstInQ<=LastInQ))
    // not ready with sorting && ZeroQueue is not empty
  { i = ZeroQueue[FirstInQ];
    FirstInQ ++;
    Order[Index] = i;
    Index ++;

      // remove arcs from i to father and mother
    f = Father[i];
    if (f!=Unknown)
    { IncArcs[f] --;
      if (IncArcs[f] == 0)
      { LastInQ ++;
        ZeroQueue[LastInQ] = f;
      }
    }
    m = Mother[i];
    if (m!=Unknown)
    { IncArcs[m] --;
      if (IncArcs[m] == 0)
      { LastInQ ++;
        ZeroQueue[LastInQ] = m;
      }
    }
  }  // while

//  PrintTopOrder (MinID, N, Index, Order);

  if (Index>N)
    return true;
  else
    return false;

}  // CheckDAG

//****************************************************************************

int ComputeMinNumberOfWomen (int N, int A0, int B0)
 // Compute the minimum number of women on a line of ancestry from A0 to B0.
 // If there is no such line, then return -1.
 // Pre: A0 != B0
{ int i, f, m,
      FirstInDEQ, LastInDEQ;
  bool B0Found;

  for (i=MinID;i<=N;i++)
    ChildToA0[i] = Unknown;

  FirstInDEQ = MaxN;  // we start in the middle of DEQueue
  LastInDEQ = MaxN;
  Distance[A0] = 0;
  DEQueue[FirstInDEQ] = A0;
  B0Found = false;

  while ((!B0Found) && (FirstInDEQ<=LastInDEQ))
   // Once we have encountered B0, we can stop, because we will never find
   // a shorter path to him/her. Even when B0 is a woman and we have
   // plenty of persons in DEQueue with a smaller Distance than she.
   // In that case, this smaller Distance will be only 1 smaller,
   // and in order to get to (woman) B0, we must add at least 1.
  {  // remove first element from DEQueue
    i = DEQueue[FirstInDEQ];
    FirstInDEQ ++;

     // add father of i to DEQueue, if we have not encountered him before
    f = Father[i];
    if ((f!=Unknown) && (ChildToA0[f]==Unknown))
    { FirstInDEQ --;
      DEQueue[FirstInDEQ] = f;
      Distance[f] = Distance[i];
      ChildToA0[f] = i;

      if (f==B0)
        B0Found = true;
    }

     // add mother of i to DEQueue, if we have not encountered her before
    m = Mother[i];
    if ((m!=Unknown) && (ChildToA0[m]==Unknown))
    { LastInDEQ ++;
      DEQueue[LastInDEQ] = m;
      Distance[m] = Distance[i]+1;
      ChildToA0[m] = i;

      if (m==B0)
        B0Found = true;
    }

  }  // while
  
  if (B0Found)
  { if (Gender[B0] == Man)
      return (Distance[B0]);
    else  // B0 is woman; we must not count her
      return (Distance[B0]-1);
  }
  else
    return -1;

}  // ComputeMinNumberOfWomen

//****************************************************************************

void PrintLine (int A0, int B0, int i)
 // Recursively print the line of ancestry from A0 to i,
 // as part of the line of ancestry from A0 to B0.
{ int child;

  if (i!=A0)
  { child = ChildToA0[i];
    PrintLine (A0, B0, child);

    if (i==Father[child])
      cout << "Father is i = " << i << EOL;
    else
      cout << "Mother is i = " << i << EOL;

    if (i==B0)
      cout << "B0 = " << B0 << EOL;
  }
  else  // i==A0
    cout << "A0 = " << A0 << EOL;

}  // PrintLine

//****************************************************************************

int main ()
{ int NInst, Inst;
  int N,
      A0, B0;
  bool OK;
  int i,
      f, m,
      Number;

  scanf ("%d\n", &NInst);
  for (Inst=1;Inst<=NInst;Inst++)
  { scanf ("%d\n", &N);
    scanf ("%d %d\n", &A0, &B0);

    OK = CheckParameters (N, A0, B0, Inst);

    if (OK)
    { for (i=MinID;i<=N;i++)
      { Father[i] = Unknown;
        Mother[i] = Unknown;
        Gender[i] = UnknownGender;
      }
    }

    for (i=1;i<=N;i++)
    { scanf ("%d %d\n", &f, &m);
        // continue reading input, even if some value was not OK
      if (( (f != Unknown) && ((f < MinID) || (f > N)) )
          ||  ( (m != Unknown) && ((m < MinID) || (m > N)) )
          ||  ( (f != Unknown) && (f == m) )
         )
      { cout << "incorrect value for f or m of person " << i
             << " at instance " << Inst << ": "
             << f << ' ' << m << EOL;
        OK = false;
      }
      if (OK)
      { if ( ( (f != Unknown) && (Gender[f] == Woman) )
            || ( (m != Unknown) && (Gender[m] == Man) )
           )
        { cout << "value for f or m of person " << i
               << " at instance " << Inst
               << " is inconsistent with earlier lines: "
               << f << ' ' << m << EOL;
          OK = false;
        }
      }

      if (OK)
      { Father[i] = f;
        Mother[i] = m;
        if (f != Unknown)
          Gender[f] = Man;
        if (m != Unknown)
          Gender[m] = Woman;
      }

    }  // for i

    if (OK)
      OK = CheckDAG (N, Inst);

    if (OK)
    {
      Number = ComputeMinNumberOfWomen (N, A0, B0);
      if (Number>=0)
      { 
//        PrintLine (A0, B0, B0);
        cout << Number << EOL;
      }
      else
        cout << "no ancestor\n";

    }  // correct input

  }  // for Inst
  
  return 0;

}
